/***************************************************************************
                          station-drag-object.h  -  description
                             -------------------
    begin                : Sun Aug 28 2005
    copyright            : (C) 2005 by Martin Witte
    email                : witte@kawo1.rwth-aachen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KRADIO_STATION_DRAG_OBJECT_H
#define KRADIO_STATION_DRAG_OBJECT_H

#include <tqdragobject.h>
#include <tdemacros.h>

class TDE_EXPORT StationDragObject : public TQStoredDrag
{
public:
    StationDragObject(const TQStringList &stationIDs, TQWidget *dragSource = NULL, const char * name = NULL);
    StationDragObject(TQWidget *dragSource = NULL, const char * name = NULL);
    virtual ~StationDragObject();

    const char *format(int i = 0) const;

    void setStations(const TQStringList &stationIDs);

    static bool canDecode (const TQMimeSource *e);
    static bool decode (const TQMimeSource *e, TQStringList &stationIDs);
};

#endif

