/***************************************************************************
                          quickbar-configuration.cpp  -  description
                             -------------------
    begin                : Son Aug 3 2003
    copyright            : (C) 2003 by Martin Witte
    email                : witte@kawo1.rwth-aachen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "quickbar-configuration.h"

using namespace std;

QuickbarConfiguration::QuickbarConfiguration (TQWidget *parent)
	: StationSelector(parent)
{
}


QuickbarConfiguration::~QuickbarConfiguration ()
{
}




#include "quickbar-configuration.moc"
