/***************************************************************************
                          utils.h  -  description
                             -------------------
    begin                : Sun Feb 3 2002
    copyright            : (C) 2002 by Martin Witte / Frank Schwanz
    email                : witte@kawo1.rwth-aachen.de / schwanz@fh-brandenburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KRADIO_UTILS_H
#define KRADIO_UTILS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

using namespace std;

#include <tdeconfig.h>
#include <tdelocale.h>
#include <kdebug.h>
#include <tqstring.h>
#define __USE_ISOC99 1
#include <math.h>

//extern const char *mixerChannelLabels[];
//extern const char *mixerChannelNames[];

extern TQString XMLEscape (const TQString &s);
TQString xmlOpenTag  (const TQString &tag, bool newline = true);
TQString xmlTag      (const TQString &tag, const TQString &s, bool newline = true);
TQString xmlTag      (const TQString &tag, int i,            bool newline = true);
TQString xmlTag      (const TQString &tag, float f,          bool newline = true);
TQString xmlCloseTag (const TQString &tag, bool newline = true);

template<class T1, class T2> inline T1 min (T1 a, T2 b) { return a < b ? a : b; }
template<class T1, class T2> inline T1 max (T1 a, T2 b) { return a < b ? b : a; }

#endif
